using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;

namespace Feel
{
    public class LiquidglasspackTracker : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel";
        public string pluginName = "LiquidglasspackTracker";
        public string pluginVersion = "1";
        public string pluginDate = "11/24" + "/2022";
        public string pluginDescription = @"Liquidglasspack Tracker";
        #endregion

        #region Vars    

        protected bool first_loading = false;

        protected JSONStorableBool IsActive;
        protected JSONStorableStringChooser _personSelectJSON;
        protected UIDynamicPopup _personDisplayPopup;
        protected JSONStorableFloat _motionFrame;
        protected JSONStorableFloat _motionSmooth;
        protected List<String> _atomBodyChoices;
        protected Atom _selectAtom;
        private Rigidbody _mouthTrigger;
        private Rigidbody _lipTrigger;
        private FreeControllerV3 _thisControl;

        Quaternion local_b;
        Quaternion local_r;

        private float _lerp_value = 0f;

        private Atom _thisAtom;

        #endregion

        public override void Init()
        {
            try
            {
                _thisControl = containingAtom.freeControllers.First(fc => fc.name == "control");

                IsActive = new JSONStorableBool("Active", false, SetTargetActive);
                RegisterBool(IsActive);
                UIDynamicToggle toggle2 = CreateToggle(IsActive, false);
                toggle2.label = "Active Drink Motion";
                toggle2.height = 60;

                var btn = CreateButton("Person Update", false);

                btn.button.onClick.AddListener(() =>
                {
                    UpdateAtomData();
                });

                _personSelectJSON = new JSONStorableStringChooser("Target Choice", null, null, "", OnTargetPersonSelect);

                _personSelectJSON.storeType = JSONStorableParam.StoreType.Physical;
                RegisterStringChooser(_personSelectJSON);

                _personDisplayPopup = CreateScrollablePopup(_personSelectJSON, false);
                _personDisplayPopup.popupPanelHeight = 300f;
                _personDisplayPopup.popup.onOpenPopupHandlers += SyncPersonChoices;
                _personDisplayPopup.label = "Person:";

                _motionFrame = new JSONStorableFloat("Motion Length", 0.35f, OnMotionFrameChanged, 0f, 1f);
                RegisterFloat(_motionFrame);
                CreateSlider(_motionFrame, false);
                _motionSmooth = new JSONStorableFloat("Motion Smooth", 1f, OnMotionSmoothChanged, 0.01f, 1f);
                RegisterFloat(_motionSmooth);
                CreateSlider(_motionSmooth, false);

                UpdateAtomData();

                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Liquidglasspack Tracker</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Control the animation using Liquidglasspack.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 200.0f;

                var btn3 = CreateButton("Go To Support", true);
                btn3.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        void FixedUpdate()
        {
            if (IsActive.val)
            {
                try
                {
                    Vector3 targetPosition;
                    Vector3 targetUpDirection;
                    Transform tempTransform;
                    Quaternion targetRotation;
                    //Quaternion targetRotationInverse;

                    targetPosition = _lipTrigger.transform.position;
                    targetUpDirection = _lipTrigger.transform.up;
                    targetRotation = _lipTrigger.transform.rotation;
                    //targetRotationInverse = _lipTrigger.transform.rotation;

                    float motion_frame = _motionFrame.val;
                    _lerp_value += _motionSmooth.val;
                    if (_lerp_value > 1f) _lerp_value = 1f;
                    //_lerp_value = 1f;

                    float height = motion_frame * 0.1f + 0.1f;
                    float n_mf = motion_frame - 0.6f;
                    if (n_mf < 0f) n_mf = 0f;
                    n_mf = n_mf / 0.4f;
                    if (n_mf > 1f) n_mf = 1f;
                    Vector3 offsetPosition = new Vector3(0f, -0.015f - (n_mf * 0.25f), 0.23f + (motion_frame * 0.1f));
                    //Vector3 offsetPosition = new Vector3(0f, -0.01f, 0.23f + (motion_frame * 0.05f));

                    tempTransform = _thisControl.transform;
                    Vector3 local_p = _lipTrigger.transform.InverseTransformPoint(targetPosition);
                    local_p = local_p + offsetPosition;
                    
                    tempTransform.transform.position = Vector3.Lerp(tempTransform.transform.position, _lipTrigger.transform.TransformPoint(local_p), _lerp_value);
                    tempTransform.rotation = Quaternion.Slerp(tempTransform.rotation, _lipTrigger.transform.rotation * local_r, _lerp_value);
                    

                }
                catch (Exception e)
                {
                    SuperController.LogError("Exception caught: " + e);
                }
            }
        }

        void OnDestroy()
        {
            
        }

        protected void SetTargetActive(JSONStorableBool jb)
        {
            if(jb.val)
            {
                local_b = Quaternion.Euler(new Vector3(0f, 0f, 0f));
                //local_r = local_r * Quaternion.Euler(new Vector3(0f, 90f, 0f));
                OnMotionFrameChanged(_motionFrame.val);
            }
        }

        private void UpdateAtomData()
        {
            _atomBodyChoices = GetAtomListChoices();
            _personSelectJSON.choices = _atomBodyChoices;
        }

        protected List<string> GetAtomListChoices()
        {
            List<string> atomChoices = new List<string>();
            Atom atom;
            foreach (string atomUID in SuperController.singleton.GetAtomUIDs())
            {
                atom = SuperController.singleton.GetAtomByUid(atomUID);
                JSONStorable geometry = atom.GetStorableByID("geometry");
                if (geometry != null)
                {
                    atomChoices.Add(atomUID);
                    //SuperController.LogMessage(">>" + atomUID);
                }
            }
            return atomChoices;
        }

        protected void OnTargetPersonSelect(string target_name)
        {

            //if (_targetBodyChoices.Count > 0) OnSelectTargetObj(target_name);
            //SuperController.LogMessage(">> OnTargetPersonSelect " + target_name);

            if(target_name != null && target_name != "")
            {
                _selectAtom = SuperController.singleton.GetAtomByUid(target_name);
                _mouthTrigger = _selectAtom.rigidbodies.First(rb => rb.name == "MouthTrigger");
                _lipTrigger = _selectAtom.rigidbodies.First(rb => rb.name == "LipTrigger");
            }

        }

        protected void SyncPersonChoices()
        {
            //SuperController.LogMessage(">> SyncPersonChoices ");
        }

        protected void OnMotionFrameChanged(float value)
        {
            _lerp_value = 0f;
            //float angle = -75f + (90f - (90f * (1f - value)));
            float n_value = value - 0.6f;
            if (n_value < 0f) n_value = 0f;
            n_value = n_value / 0.4f;
            if (n_value > 1f) n_value = 1f;
            float angle = -90f + (90f - (90f * (1f - n_value)));
            //angle = -90f;
            local_r = local_b * Quaternion.Euler(new Vector3(angle, 0f, 0f));
            //local_r = local_b * Quaternion.Euler(new Vector3(0f, 0f, 0f));
            //local_r = local_b;
        }

        protected void OnMotionSmoothChanged(float value)
        {
            
        }
    }

}